package com.perforce.polarion.element.event;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.perforce.polarion.element.field.WorkItemField;
import com.polarion.alm.tracker.model.IWorkItem;
import com.polarion.platform.persistence.IDataService;
import com.polarion.platform.persistence.diff.IDiffManager;
import com.polarion.platform.persistence.diff.IFieldDiff;
import com.polarion.platform.persistence.events.IPersistenceEvent;
import com.polarion.platform.persistence.events.IPersistenceListener;
import com.polarion.platform.persistence.model.IPObject;
import com.polarion.platform.persistence.notifications.ChangeInfo;
import com.polarion.subterra.base.data.identification.IContextId;

public class P4PersistenceListener implements IPersistenceListener {

	private static final Logger log = Logger.getLogger(P4PersistenceListener.class);

	private final IDataService dataService;

	public P4PersistenceListener(IDataService dataService) {
		super();
		this.dataService = dataService;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void handleEvent(IPersistenceEvent event) {

		String revision = event.getRevision();
		String previousRevision = event.getPreviousRevision();

		// CREATED
		for (IPObject pobject : (List<IPObject>) event.getCreated()) {
			ChangeInfo change = ChangeInfo.create(pobject, revision, previousRevision, false, true);
			handleCreation(change);
			change.forget();
		}

		// MODIFIED
		for (IPObject pobject : (List<IPObject>) event.getModified()) {
			ChangeInfo change = ChangeInfo.create(pobject, revision, previousRevision, true, true);
			handleModification(change);
			change.forget();
		}

		// DELETED
		for (IPObject pobject : (List<IPObject>) event.getDeleted()) {
			ChangeInfo change = ChangeInfo.create(pobject, revision, previousRevision, true, false);
			handleRemoval(change);
			change.forget();
		}
	}

	private void handleCreation(ChangeInfo change) {
		log.info("Creation of " + change.current.getUri());

		// exit early if not a WorkItem
		if (!(change.current instanceof IWorkItem)) {
			return;
		}

		IWorkItem workItem = (IWorkItem) change.current;

		Map<WorkItemField, String> changes = new HashMap<>();
		for (WorkItemField item : WorkItemField.values()) {
			String value = item.get(workItem);
			changes.put(item, value);
		}

		IContextId context = workItem.getContextId();
		String id = WorkItemField.ID.get(workItem);
		JobEvent jobEvent = new JobEvent(context);
		jobEvent.create(id, changes);
	}

	private void handleRemoval(ChangeInfo change) {
		log.info("Removal of " + change.current.getUri());

		// exit early if not a WorkItem
		if (!(change.current instanceof IWorkItem)) {
			return;
		}

		IWorkItem workItem = (IWorkItem) change.current;

		IContextId context = workItem.getContextId();
		String id = WorkItemField.ID.get(workItem);
		JobEvent jobEvent = new JobEvent(context);
		jobEvent.delete(id);
	}

	private void handleModification(ChangeInfo change) {
		log.info("Modification of " + change.current.getUri());

		// exit early if not a WorkItem
		if (!(change.current instanceof IWorkItem)) {
			return;
		}

		IWorkItem workItem = (IWorkItem) change.current;

		IDiffManager diffManager = dataService.getDiffManager();
		IFieldDiff[] fieldDiffs = diffManager.generateDiff(change.before, change.after, IWorkItem.DIFF_IGNORED_FIELDS);

		Map<WorkItemField, String> changes = new HashMap<>();
		for (IFieldDiff fieldDiff : fieldDiffs) {
			String name = fieldDiff.getFieldName();
			WorkItemField workItemField = WorkItemField.parse(name);

			// Skip unsupported fields
			if (workItemField == null) {
				continue;
			}

			String value = workItemField.get(workItem);
			changes.put(workItemField, value);
		}

		IContextId context = workItem.getContextId();
		String id = WorkItemField.ID.get(workItem);
		JobEvent jobEvent = new JobEvent(context);
		jobEvent.update(id, changes);
	}

}
